//-----------------------------------------------------------------------------
//
//			      XMM Science Analysis System (SAS)
//
//-----------------------------------------------------------------------------
//
//	EnergyCombinator --- A class to combine energy values into PHA for EMOS
//
//	DESCRIPTION:
//		Instances of class EnergyCombine can combine the four energies
//		values of an EMOS-detected event into a single PHA value.
//
//	CHANGE HISTORY:
//		UL, 1999-11-19: 1.0.0: - framework (to be taken over by Michel
//						    Lortholary/Jean Ballet)
//
//	AUTHOR:
//		U. Lammers, Fri Nov 19 15:35:09 MET 1999
//
//	REFERENCES:
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/calmosalgo/src/EnergyCombinator.h,v 1.4 2000/05/22 07:35:46 ulammers Exp $
//

#ifndef ENERGY_COMBINE_H
#define ENERGY_COMBINE_H

#include <vector>
#include "CalServer.h"
#include "CalDataVector.h"


//
//	---( EnergyCombinator )-----------------------------------------------
//
class EnergyCombinator {
public:
	EnergyCombinator(const vector<float> &e1Coefs,
			 const vector<float> &e2Coefs,
			 const vector<float> &e3Coefs,
			 const vector<float> &e4Coefs)
	  : _e1Coefs(e1Coefs), _e2Coefs(e2Coefs), _e3Coefs(e3Coefs),
	    _e4Coefs(e4Coefs) { PROC  ENDPROC }

//	PURPOSE:
//		Construct EnergyCombinator object
//
//	ARGUMENTS:
//		e1/2/3/4Coefs	: vector with coefficients to control
//				  the energy combination; one coefficient
//				  per possible event pattern, i.e.
//				  e1Coefs.size() gives the number of
//				  patterns

	virtual CalReal32Vector &combine(const CalReal32Vector &energye1,
					 const CalInt16Vector &energye2,
					 const CalInt8Vector &pattern,
					 CalReal32Vector &pha,
					 const CalReal32Vector &locbkg ,
					 const CalInt16Vector &energye3 ,
					 const CalInt16Vector &energye4 ,
					 const CalInt8Vector &peripix ,
					 const CalInt32Vector &flag) ;

	virtual CalReal32Vector &combine(const CalReal32Vector &energye1,
					 const CalInt16Vector &energye2,
					 const CalInt8Vector &pattern,
					 CalReal32Vector &pha) ;

	virtual CalReal32Vector &combine(const CalReal32Vector &energye1,
					 const CalInt16Vector &energye2,
					 const CalInt8Vector &pattern,
					 CalReal32Vector &pha,
					 const CalReal32Vector &locbkg) ;


	virtual CalReal32Vector &combine(const CalReal32Vector &energye1,
					 const CalInt16Vector &energye2,
					 const CalInt8Vector &pattern,
					 CalReal32Vector &pha,
					 const CalInt16Vector &energye3,
					 const CalInt16Vector &energye4,
					 const CalInt8Vector &peripix ,
					 const CalInt32Vector &flag) ;

//	PURPOSE:
//		Combine four energy values into a PHA value for
//		a vector of events
//
//	ARGUMENTS:
//		In:
//			energye1	: 
//			energye2	: 
//			energye3	: optional 
//			energye4	: optional  
//			pattern		:
//			peripix		: optional  
//			flag		: optional 
//			locbkg		: optional  
//		Out:
//			pha		: 
//
//	RETURNS:
//		<pha> vector

	virtual ~EnergyCombinator() { PROC ENDPROC }

//	PURPOSE:
//		Destroy EnergyCombinator constructed with above constructor

private:
//	data members
//      vector with combination coefficients
	vector<float> _e1Coefs, _e2Coefs,
	              _e3Coefs, _e4Coefs;

        void AlgoCombine(const CalReal32Vector &energye1,
			 const CalInt16Vector &energye2,
			 const CalInt8Vector &pattern,
			 CalReal32Vector &pha,    // out
			 const CalReal32Vector &locbkg, 
			 const CalInt16Vector &energye3,
			 const CalInt16Vector &energye4,
			 const CalInt8Vector &peripix,
			 const CalInt32Vector &flag,
			 bool withLocbkg,
			 bool fulldata   );

};

#endif
